# -*- coding: utf-8 -*-\
import logging

from oauthlib.oauth1.rfc5849 import signature
from oauthlib.oauth1 import ResourceEndpoint
from talons import exc
from talons.auth import interfaces

LOG = logging.getLogger(__name__)


class Identifier(interfaces.Identifies):
    def identify(self, request):
        if request.env.get(self.IDENTITY_ENV_KEY) is not None:
            return True

        if self._is_oauth(request):
            identity = OAuthIdentity(
                request.url,
                request.method,
                request.stream,
                request.headers,
            )
            request.env[self.IDENTITY_ENV_KEY] = identity
            return True

        return False

    @staticmethod
    def _is_oauth(request):
        check = signature.collect_parameters
        return any((
            check(headers=request.headers,
                  exclude_oauth_signature=False, with_realm=True),
            check(body=request.stream,
                  exclude_oauth_signature=False),
            check(uri_query=request.query_string,
                  exclude_oauth_signature=False)
        ))


class Authenticator(interfaces.Authenticates):
    def __init__(self, **conf):
        """ Construct a concrete object with a set of keyword configuration
        options.

        :param **conf:

            oauth_validator: RequestValidator for oauthlib

        :raises `talons.exc.BadConfiguration` if configuration options
            are not valid or conflict with each other.
        """
        oauth_validator = conf.pop('oauth1_validator', None)
        if not oauth_validator:
            msg = ("Missing required oauth_validator "
                   "configuration option.")
            LOG.error(msg)
            raise exc.BadConfiguration(msg)

        self.provider = ResourceEndpoint(oauth_validator)
        self.realms = conf.pop('oauth1_realms', [])

    def authenticate(self, identity):
        """ Authenticate user client using oauthlib's ResourceEndopoint
        """
        v, r = self.provider.validate_protected_resource_request(
            # we use gettattr because we depend on overriden Identity
            # - other identities can lack these attributes
            getattr(identity, 'url', None),
            http_method=getattr(identity, 'method', None),
            body=getattr(identity, 'stream', None),
            headers=getattr(identity, 'headers', None),
            realms=self.realms,
        )
        return bool(v)


class OAuthIdentity(interfaces.Identity):
    def __init__(self, url, method, stream, headers):
        self.url = url
        self.method = method
        self.stream = stream
        self.headers = headers

        super(OAuthIdentity, self).__init__(None)



# -*- coding: utf-8 -*-
import logging

import falcon
import fixtures
import mock
import oauthlib
from oauthlib.oauth1 import SIGNATURE_TYPE_AUTH_HEADER
import testtools
from talons.auth.oauth.oauth1 import Authenticator, OAuthIdentity, Identifier

from .validator import ExampleRequestValidator
from . import validator

LOG_FORMAT = "[%(levelname)-7s] %(msg)s"


class TestOAuth(testtools.TestCase):
    def setUp(self):
        self.useFixture(fixtures.FakeLogger(level=logging.DEBUG,
                                            format=LOG_FORMAT))
        super(TestOAuth, self).setUp()

    def get_oauth_client(self, client_key):
        access_token = validator.ACCESS_TOKENS[client_key]
        return oauthlib.oauth1.Client(
            client_key,
            client_secret=validator.CLIENT_SECRETS[client_key],
            resource_owner_key=access_token['token'],
            resource_owner_secret=access_token['secret'],
            signature_type=SIGNATURE_TYPE_AUTH_HEADER,
            signature_method=oauthlib.oauth1.SIGNATURE_PLAINTEXT,
        )

    def _signed_request(self, url, method):
        return self.oauth_client.sign(url, method)

    def test_identity_already_exists(self):
        req = mock.MagicMock()
        req.env = mock.MagicMock()
        req.env.get = mock.MagicMock()
        req.env.get.side_effect = ['something']

        i = Identifier()
        i.identify(req)

        req.env.get.assert_called_once_with('wsgi.identity')

    def test_identity_invalid(self):
        req = mock.MagicMock(spec=falcon.Request)

        type(req).stream = stream = mock.PropertyMock(return_value=None)
        type(req).headers = headers = mock.PropertyMock(return_value=dict())
        type(req).query_string = query = mock.PropertyMock(return_value="")

        type(req).method = mock.PropertyMock(return_value="GET")
        type(req).url = mock.PropertyMock(return_value='resource_url/')

        type(req).env = env = mock.PropertyMock(return_value=dict())

        mod_cls = 'talons.auth.oauth.oauth1.OAuthIdentity'
        with mock.patch(mod_cls, spec=OAuthIdentity) as i_mock:
            i = Identifier()
            i.identify(req)

            # this three must me used to check if it could be OAuth
            stream.assert_called_once_with()
            headers.assert_called_once_with()
            query.assert_called_once_with()

            assert env.called
            assert not i_mock.called

    def test_identity_valid(self):
        req = mock.MagicMock(spec=falcon.Request)

        method = "GET"
        url, headers, body = \
            self.get_oauth_client(validator.CLIENT_KEYS[0]).sign(
                'https://example.com/resource', method)

        type(req).stream = stream = mock.PropertyMock(return_value=None)
        type(req).method = method = mock.PropertyMock(return_value=method)
        type(req).headers = headers = mock.PropertyMock(return_value=headers)

        type(req).query_string = mock.PropertyMock(return_value="")
        type(req).url = url = mock.PropertyMock(return_value=url)

        # clean env to ensure that won't stop checking for identity
        type(req).env = mock.PropertyMock(return_value=dict())

        mod_cls = 'talons.auth.oauth.oauth1.OAuthIdentity'
        with mock.patch(mod_cls, spec=OAuthIdentity):
            i = Identifier()
            i.identify(req)

            assert stream.called
            assert method.called
            assert headers.called
            assert url.called
            assert req.env[Identifier.IDENTITY_ENV_KEY]

    def test_authenticator_invalid(self):
        """ Test authentication fail when request does not identify
        """
        stream = "example body"
        method = "GET"
        headers = {}
        url = "resource"

        identity = OAuthIdentity(
            url, method=method, headers=headers, stream=stream)

        authenticator = Authenticator(
            oauth1_validator=ExampleRequestValidator())
        assert not authenticator.authenticate(identity)

    def test_authenticator_valid_realms_match(self):
        """ Test authentication success when realms and token are valid
        """
        method = "GET"

        # CLIENT_KEYS[0] is a consumer with 'photos', 'printers' realms
        url, headers, body = \
            self.get_oauth_client(validator.CLIENT_KEYS[0]).sign(
                'https://example.com/resource', method)

        identity = OAuthIdentity(
            url, method=method, headers=headers, stream=body
        )

        authenticator = Authenticator(
            oauth1_validator=ExampleRequestValidator(),
            oauth1_realms=['photos', 'printers'])
        assert authenticator.authenticate(identity)

    def test_authenticator_invalid_realm(self):
        """ Test authentication fail when access token is valid but
        it has no required realms
        """
        method = "GET"
        # CLIENT_KEYS[1] is a consumer with 'foo', 'bar' realms
        url, headers, body = \
            self.get_oauth_client(validator.CLIENT_KEYS[1]).sign(
                'https://example.com/resource', method)

        identity = OAuthIdentity(
            url, method=method, headers=headers, stream=body,
        )

        authenticator = Authenticator(
            oauth1_validator=ExampleRequestValidator(),
            oauth1_realms=['photos', 'printers'],
        )
        assert not authenticator.authenticate(identity)

    def test_authenticator_no_realms(self):
        """ Test authentication fail when consumer has access token without
        any realms
        """
        method = "GET"

        # CLIENT_KEYS[2] is a consumer with no realms
        url, headers, body = \
            self.get_oauth_client(validator.CLIENT_KEYS[2]).sign(
                'https://example.com/resource', method)

        identity = OAuthIdentity(
            url, method=method, headers=headers, stream=body,
        )

        authenticator = Authenticator(
            oauth1_validator=ExampleRequestValidator(),
            oauth1_realms=['photos', 'printers'],
        )
        assert not authenticator.authenticate(identity)

    def test_authenticator_valid_no_realms_required(self):
        """ Test authentication success when realms are not required
        """
        method = "GET"

        # CLIENT_KEYS[0] is a consumer with 'photos', 'printers' realms
        url, headers, body = \
            self.get_oauth_client(validator.CLIENT_KEYS[0]).sign(
                'https://example.com/resource', method)

        identity = OAuthIdentity(
            url, method=method, headers=headers, stream=body
        )

        authenticator = Authenticator(
            oauth1_validator=ExampleRequestValidator(),
            oauth1_realms=[])
        assert authenticator.authenticate(identity)



# -*- coding: utf-8 -*-
""" Testing RequestValidator for oauthlib
"""
from oauthlib.common import UNICODE_ASCII_CHARACTER_SET
from oauthlib.oauth1 import RequestValidator

CLIENT_KEYS = [u'test_client_key',
               u'test_restricted_client_invalid_realms',
               u'test_restricted_client_no_realms']

CLIENT_SECRETS = {
    CLIENT_KEYS[0]: u"test_secret",
    CLIENT_KEYS[1]: u"test_secret",
    CLIENT_KEYS[2]: u"test_secret",
}

CLIENT_RSAS = {
    CLIENT_KEYS[0]: u'test RSA key',
    CLIENT_KEYS[1]: u'test RSA key',
    CLIENT_KEYS[2]: u"test RSA key",
}

REQUEST_TOKENS = {
    CLIENT_KEYS[0]: {
        'token': u'test request token0',
        'secret': u'test token secret',
        'realms': [u'photos', u'printers'],
        'redirect_uri': u'oob'
    },
    CLIENT_KEYS[1]: {
        'token': u'test request token1',
        'secret': u'test token secret',
        'realms': [u'foo', u'bar'],
        'redirect_uri': u'oob'
    },
}

ACCESS_TOKENS = {
    CLIENT_KEYS[0]: {
        'token': u'test_access_token0',
        'secret': u'test_secret',
        'realms': [u'photos', u'printers'],
    },
    CLIENT_KEYS[1]: {
        'token': u'test access token1',
        'secret': u'test token secret',
        'realms': [u'foo', u'bar'],
    },
    CLIENT_KEYS[2]: {
        'token': u'test access token2',
        'secret': u'test token secret',
        'realms': [],
    },
}
NONCES = []
VERIFIERS = {
    REQUEST_TOKENS[CLIENT_KEYS[0]]['token']: 'test verifier'
}


class ExampleRequestValidator(RequestValidator):
    dummy_client = 'dummy'
    dummy_request_token = 'dummy'
    dummy_access_token = 'dummy'

    default_lengths = (1, 100)

    client_key_length = default_lengths
    request_token_length = default_lengths
    access_token_length = default_lengths
    nonce_length = default_lengths
    verifier_length = default_lengths
    safe_characters = set(UNICODE_ASCII_CHARACTER_SET + '_')

    def validate_client_key(self, client_key, request):
        """ Needed by: AccessTokenEndpoint, RequestTokenEndpoint,
                       ResourceEndpoint
        """
        return client_key in CLIENT_KEYS

    def validate_request_token(self, client_key, token, request):
        """ Needed by: AccessTokenEndpoint
        """
        try:
            return REQUEST_TOKENS[client_key]['token'] == token
        except KeyError:
            return False

    def validate_access_token(self, client_key, token, request):
        """ Needed by: ResourceEndpoint
        """
        try:
            return ACCESS_TOKENS[client_key]['token'] == token
        except KeyError:
            return False

    def validate_timestamp_and_nonce(self, client_key, timestamp, nonce,
                                     request, request_token=None,
                                     access_token=None):
        """ Needed by: AccessTokenEndpoint, RequestTokenEndpoint
                       ResourceEndpoint
        """
        return ((client_key, timestamp, nonce, request_token or access_token)
                not in NONCES)

    def validate_redirect_uri(self, client_key, redirect_uri, request):
        """ Needed by: RequestTokenEndpoint
        """
        # we do not care since it's for purpose of tests
        return True

    def validate_requested_realms(self, client_key, realms, request):
        """ Needed by: RequestTokenEndpoint
        """
        # we do not care since it's for purpose of tests
        return True

    def validate_realms(self, client_key, token, request, uri=None,
                        realms=None):
        """ Needed by: ResourceEndpoint
        """
        return set(self.get_realms(token, request)).issuperset(set(realms))

    def validate_verifier(self, client_key, token, verifier, request):
        """ Needed by: AccessTokenEndpoint
        """
        try:
            return VERIFIERS[token] == verifier
        except KeyError:
            return False

    def get_client_secret(self, client_key, request):
        """ Needed by: AccessTokenEndpoint, RequestTokenEndpoint,
                       ResourceEndpoint
        """
        return CLIENT_SECRETS.get(client_key, 'dummy')

    def get_request_token_secret(self, client_key, token, request):
        """ Needed by: AccessTokenEndpoint
        """
        return REQUEST_TOKENS.get(client_key, {'secret': 'dummy'})['secret']

    def get_access_token_secret(self, client_key, token, request):
        """ Needed by: AccessTokenEndpoint"""
        return ACCESS_TOKENS.get(client_key, {'secret': 'dummy'})['secret']

    def get_rsa_key(self, client_key, request):
        """ Needed by: AccessTokenEndpoint, RequestTokenEndpoint,
                       ResourceEndpoint
        """
        return CLIENT_RSAS.get(client_key, "dummy")

    def save_request_token(self, token, request):
        """ Needed by: RequestTokenEndpoint
        """

        REQUEST_TOKENS[request.client_key] = {
            'token': token['oauth_token'],
            'secret': token['oauth_secret'],
        }

    def save_verifier(self, token, verifier, request):
        """ Needed by: AuthorizationEndpoint
        """
        VERIFIERS[token] = verifier

    def save_access_token(self, token, request):
        """ Needed by: AccessTokenEndpoint
        """
        ACCESS_TOKENS[request.client_key] = {
            'token': token['oauth_token'],
            'secret': token['oauth_secret'],
        }

    def get_realms(self, token, request):
        """ Get allowed realms for given access token
        Needed by: AuthorizationEndpoint, AccessTokenEndpoint
        """
        access_tokens = [token_dict for token_dict in ACCESS_TOKENS.values()
                         if token_dict['token'] == token]
        try:
            realms = access_tokens.pop()['realms']
        except IndexError:
            realms = []
        return realms

    def get_redirect_uri(self, token, request):
        """ Needed by: RequestTokenUri
        """
        return token['redirect_uri']

    def invalidate_request_token(self, client_key, request_token, request):
        """ Needed by: AccessTokenEndpoint
        """
        # do not care
        pass

    def verify_realms(self, token, realms, request):
        """ Needed by: AuthorizationEndpoint
        """
        valid_realms = self.get_realms(token)
        return set(valid_realms) == set(realms)

    def verify_request_token(self, token, request):
        """ Needed by: AuthorizationEndpoint
        """
        all_tokens = [value['token'] for value in REQUEST_TOKENS.values()]
        return token in all_tokens

    def get_default_realms(self, client_key, request):
        """ Needed by: RequestTokenEndpoint
        """
        return []



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os

PACKAGES = find_packages(exclude='tests')
README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()


def strip_comments(l):
    return l.split('#', 1)[0].strip()


def reqs(*f):
    return list(filter(None, [strip_comments(l) for l in open(
        os.path.join(os.getcwd(), *f)).readlines()]))


def get_version(version_tuple):
    if not isinstance(version_tuple[-1], int):
        return '.'.join(map(str, version_tuple[:-1])) + version_tuple[-1]
    return '.'.join(map(str, version_tuple))


init = os.path.join(
    os.path.dirname(__file__),
    'talons', 'auth', 'oauth', '__init__.py'
)

version_line = list(filter(lambda l: l.startswith('VERSION'), open(init)))[0]

VERSION = get_version(eval(version_line.split('=')[-1]))
INSTALL_REQUIRES = reqs('requirements.txt')

setup(
    name='talons.auth.oauth',
    version=VERSION,
    author='Michał Jaworski',
    author_email='swistakm@gmail.com',
    description='OAuth 1.0 extension for Talons WSGI middleware library',
    long_description=README,
    url="https://github.com/swistakm/talons-oauth",

    packages=PACKAGES,

    include_package_data=True,
    install_requires=INSTALL_REQUIRES,
    zip_safe=True,

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
)
